/*
 * Name:    mx_util.c
 *
 * Purpose: Define some of the utility functions.
 *
 * Author:  William Lavender
 *
 *------------------------------------------------------------------------
 *
 * Copyright 1999-2000 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <time.h>
#include <float.h>
#include <math.h>
#include <signal.h>

#include "mx_osdef.h"

#ifndef OS_MPW
#include <sys/types.h>
#endif

#if defined( OS_WIN32 )
#include <winsock.h>
#include <winerror.h>
#endif

#if defined( OS_UNIX )
#include <pwd.h>
#endif

#include "mxunistd.h"

#include "mx_util.h"
#include "mx_rec.h"

/*-------------------------------------------------------------------------*/

MX_EXPORT char *
mx_username( char *buffer, size_t max_buffer_length )
{
#if !defined( OS_SOLARIS2 ) && !defined( OS_MACOSX )
	const char fname[] = "mx_username()";
#endif

	static char local_buffer[ MXU_USERNAME_LENGTH + 1 ];
	char *ptr;

	if ( buffer == NULL ) {
		ptr = local_buffer;
		max_buffer_length = MXU_USERNAME_LENGTH;
	} else {
		ptr = buffer;
	}

	/* Getting the username varies from OS to OS, so I will fill this in
	 * as time goes on.  Note that the various versions of Unix do not
	 * seem to be able to agree with each other on how to reentrantly
	 * extend getpwuid().
	 */

#if defined( OS_LINUX ) || defined( OS_HPUX )
	{
		char scratch_buffer[ 512 ];
		struct passwd pw_buffer, *pw;
		uid_t uid;
		int status, saved_errno;

		pw = NULL;

		uid = getuid();

#if defined( OS_LINUX )
		status = getpwuid_r( uid, &pw_buffer, scratch_buffer,
					sizeof(scratch_buffer), &pw );
#elif defined( OS_SOLARIS2 )
		/* This doesn't work for some reason. */

		MX_DEBUG(-2,("%s: before getpwuid_r()", fname));

		pw = getpwuid_r( uid, &pw_buffer, scratch_buffer,
					sizeof(scratch_buffer) );

		if ( pw == NULL ) {
			status = -1;
		} else {
			status = 0;
		}

		MX_DEBUG(-2,("%s: pw = , status = %d",
			fname, status));
#elif defined( OS_HPUX )
		status = getpwuid_r( uid, &pw_buffer, scratch_buffer,
					sizeof(scratch_buffer) );

		pw = &pw_buffer;
#endif

		if ( status != 0 ) {
			saved_errno = errno;

			(void) mx_error( MXE_ILLEGAL_ARGUMENT, fname,
	"Cannot determine user name for UID %lu, setting it to 'unknown'.  "
	"Error code = %d, error text = '%s'", (unsigned long) uid,
				saved_errno, strerror( saved_errno ) );

			mx_strncpy( ptr, "unknown", max_buffer_length );

			return ptr;
		}
		mx_strncpy( ptr, pw->pw_name, max_buffer_length );
	}

/* End of OS_LINUX, OS_SOLARIS2, and OS_HPUX section. */

#elif defined( OS_SGI ) || defined( OS_SUNOS4 ) || defined( OS_SOLARIS2 ) \
   || defined( OS_MACOSX )
	{
		/* This method is not reentrant. */

		struct passwd *pw;

		pw = getpwuid( getuid() );

		if ( pw == NULL ) {
			mx_strncpy( ptr, "unknown", max_buffer_length );

			return ptr;
		}
		mx_strncpy( ptr, pw->pw_name, max_buffer_length );
	}
#elif defined( OS_WIN32 )
	{
		TCHAR message_buffer[MXU_ERROR_MESSAGE_LENGTH - 120];
		DWORD dword_max_buffer_length;
		DWORD last_error_code;
		BOOL status;

		dword_max_buffer_length = max_buffer_length;

		/* I don't yet know which of the following two methods
		 * is better.
		 */
#if 0
		status = WNetGetUser( NULL, ptr, &dword_max_buffer_length );

		if ( status != NO_ERROR ) {
#else
		status = GetUserName( ptr, &dword_max_buffer_length );

		if ( status == 0 ) {
#endif
			last_error_code = GetLastError();

			switch ( last_error_code ) {
			case ERROR_NOT_LOGGED_ON:
				(void) mx_error( MXE_FUNCTION_FAILED, fname,
				"The current user is not logged into the "
				"network.  Username = 'unknown'." );

				break;
			default:
				mx_win32_error_message( last_error_code,
				    message_buffer, sizeof(message_buffer) );

				(void) mx_error( MXE_FUNCTION_FAILED, fname,
				"Cannot get username for the current thread.  "
				"Win32 error code = %ld, error message = '%s'",
				    last_error_code, message_buffer );

				break;
			}
			mx_strncpy( ptr, "unknown", max_buffer_length );
		}
	}

#elif defined( OS_VMS )

	mx_strncpy( ptr, "vms", max_buffer_length );

#elif defined( OS_MPW )

	mx_strncpy( ptr, "mac", max_buffer_length );

#elif defined( OS_DJGPP )

	/* A kludge to keep GCC from complaining about fname being unused. */

	*ptr = fname[0];

	mx_strncpy( ptr, "djgpp", max_buffer_length );

#else

#error mx_username() has not been implemented for this platform.

#endif
	return ptr;
}

/*-------------------------------------------------------------------------*/

MX_EXPORT void
mx_standard_signal_error_handler( int signal_number )
{
	static char signal_name[80];

	switch ( signal_number ) {
#ifdef SIGILL
	case SIGILL:
		strcpy( signal_name, "SIGILL (illegal instruction)" );
		break;
#endif
#ifdef SIGTRAP
	case SIGTRAP:
		strcpy( signal_name, "SIGTRAP" );
		break;
#endif
#ifdef SIGIOT
	case SIGIOT:
		strcpy( signal_name, "SIGIOT (I/O trap)" );
		break;
#endif
#ifdef SIGBUS
	case SIGBUS:
		strcpy( signal_name, "SIGBUS (bus error)" );
		break;
#endif
#ifdef SIGFPE
	case SIGFPE:
		strcpy( signal_name, "SIGFPE (floating point exception)" );
		break;
#endif
#ifdef SIGSEGV
	case SIGSEGV:
		strcpy( signal_name, "SIGSEGV (segmentation violation)" );
		break;
#endif
	default:
		sprintf( signal_name, "%d", signal_number );
		break;
	}

	mx_info( "CRASH: This program has died due to signal %s.",
			signal_name );

	/* Print out the stack traceback. */

	mx_stack_traceback();

	/* Try to force a core dump. */

#ifdef SIGABRT
	signal( SIGABRT, SIG_DFL );

	abort();
#else
	exit(1);
#endif
}

/*-------------------------------------------------------------------------*/

MX_EXPORT int
mx_get_max_file_descriptors( void )
{
	int result;

	/* This operation varies from OS to OS, so I will fill this in as time
	 * goes on.
	 */

#if defined( OS_LINUX ) || defined( OS_SOLARIS2 ) || defined( OS_SGI ) \
	|| defined( OS_HPUX ) || defined( OS_SUNOS4 ) || defined( OS_MACOSX )

	result = getdtablesize();

#elif defined( OS_WIN32 )

	result = FD_SETSIZE;

#elif defined( OS_VMS ) || defined( OS_MPW )

	result = 16;		/* A wild guess and probably wrong. */

#elif defined( OS_DJGPP )

	result = 64;

#else

#error mx_get_max_file_descriptors() has not been implemented for this platform.

#endif

	return result;
}

MX_EXPORT long
mx_round( double value )
{
	long result;

	if ( value >= 0.0 ) {
		result = (long) ( 0.5 + value );
	} else {
		result = (long) ( -0.5 + value );
	}

	return result;
}

MX_EXPORT long
mx_round_up( double value, double threshold )
{
	long result;

	if ( value >= 0.0 ) {
		result = (long) ( value + 1.0 - threshold );
	} else {
		result = (long) ( value - 1.0 + threshold );
	}

	return result;
}

MX_EXPORT long
mx_round_down( double value, double threshold )
{
	long result;

	if ( value >= 0.0 ) {
		result = (long) ( value + threshold );
	} else {
		result = (long) ( value - threshold );
	}

	return result;
}

/*
 * mx_divide_safely divides two floating point numbers by each other
 * while testing for and avoiding division by zero.
 */

#define MX_WARN_ABOUT_DIVISION_BY_ZERO	FALSE

MX_EXPORT double
mx_divide_safely( double numerator, double denominator )
{
	double result;

	if ( fabs(denominator) >= 1.0 ) {
		result = numerator / denominator;
	} else {
		if ( fabs(numerator) < fabs(denominator * DBL_MAX) ) {
			result = numerator / denominator;
		} else {
			if ( (numerator >= 0.0 && denominator >= 0.0)
			  || (numerator < 0.0 && denominator < 0.0) )
			{
				result = DBL_MAX;
			} else {
				result = -DBL_MAX;
			}
#if MX_WARN_ABOUT_DIVISION_BY_ZERO
			mx_warning(
				"*** mx_divide_safely(): Division by zero "
				"avoided for numerator %g and denominator %g."
				"  Result set to %g. ***",
				numerator, denominator, result );
#endif
		}
	}
	return result;
}

/* mx_difference() computes a relative difference function.
 *
 * Derived from pages 250-251 in "C Programming FAQs" by Steve Summit,
 * Addison Wesley, 1996.
 */

MX_EXPORT double
mx_difference( double value1, double value2 )
{
	double abs1, abs2, maxabs, result;

	if ( value1 >= 0.0 ) {
		abs1 = value1;
	} else {
		abs1 = - value1;
	}

	if ( value2 >= 0.0 ) {
		abs2 = value2;
	} else {
		abs2 = - value2;
	}

	if ( abs1 >= abs2 ) {
		maxabs = abs1;
	} else {
		maxabs = abs2;
	}
	
	if ( maxabs == 0.0 ) {
		result = 0.0;
	} else {
		result = ( value1 - value2 ) / maxabs;

		if ( result < 0.0 ) {
			result = - result;
		}
	}

	return result;
}

/* Another function from "C Programming FAQs" is the following simple
 * wildcard matcher from page 228 which is attributed to Arjan Kenter.
 *
 * mx_match() does what I need without requiring the inclusion
 * of complicated regular expression matching functions.
 *
 * As an example, the call mx_match( "a*b.c", "aplomb.c" ) would return 1.
 */

MX_EXPORT int
mx_match( char *pattern, char *string )
{
	switch( *pattern ) {
	case '\0':	return ! *string;

	case '*':	return mx_match( pattern+1, string ) ||
				( *string && mx_match( pattern, string+1 ) );

	case '?':	return *string && mx_match( pattern+1, string+1 );

	default:	return *pattern == *string &&
					mx_match( pattern+1, string+1 );
	}
}

MX_EXPORT char *
mx_current_time_string( void )
{
	char *ptr;
	size_t length;
	time_t seconds_since_1970_started;

	seconds_since_1970_started = time(NULL);

	ptr = ctime( &seconds_since_1970_started );

	/* Zap the newline at the end of the string returned by ctime(). */

	length = strlen(ptr);

	ptr[length - 1] = '\0';

	return ptr;
}

MX_EXPORT char *
mx_skip_string_fields( char *buffer, int num_fields )
{
	char *ptr;
	int num_skipped;
	size_t segment_length;

	ptr = buffer;

	/* Skip over any leading whitespace. */

	segment_length = strspn( ptr, MX_RECORD_FIELD_SEPARATORS );

	ptr += segment_length;

	if ( *ptr == '\0' )
		return ptr;

	/* Now loop over string fields. */

	num_skipped = 0;

	while ( num_skipped < num_fields ) {

		/* Skip the string. */

		segment_length = strcspn( ptr, MX_RECORD_FIELD_SEPARATORS );

		ptr += segment_length;

		num_skipped++;

		if ( *ptr == '\0' )
			return ptr;

		/* Skip the trailing whitespace. */

		segment_length = strspn( ptr, MX_RECORD_FIELD_SEPARATORS );

		ptr += segment_length;

		if ( *ptr == '\0' )
			return ptr;
	}

	return ptr;
}

