/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "mxunistd.h"

#include "mxconfig.h"
#include "mx_util.h"
#include "mx_key.h"
#include "keycodes.h"

#include "motor.h"
#include "mdialog.h"

#define WHITESPACE	" \t"

int
motor_get_int( FILE *file, char *prompt,
		int have_default, int default_value, int *value,
		int lower_limit, int upper_limit)
{
	char buffer[100];
	char default_string[50];
	int input_accepted, status;
	int count, string_length, buffer_length, whitespace_length;

	if ( have_default ) {
		sprintf( default_string, "%d", default_value );
	} else {
		strcpy( default_string, "" );
	}

	input_accepted = FALSE;

	while ( ! input_accepted ) {

		buffer_length = sizeof(buffer) - 1;

		status = motor_get_string(file, prompt, default_string,
						&buffer_length, &buffer[0]);

		if ( status == FAILURE ) {
			return FAILURE;
		}

		string_length = strlen( buffer );

		whitespace_length = strspn( buffer, WHITESPACE );

		if ( whitespace_length >= string_length ) {
			if ( have_default ) {
				*value = default_value;
			} else {
				*value = 0;       /* The ultimate default. */
			}
		}

		count = sscanf( buffer, "%d", value );

		if ( count == 0 ) {
			fprintf( file, "Non-numeric value entered.\n" );

		} else if ( *value < lower_limit || *value > upper_limit ) {
			fprintf( file,
			"Value %d is outside the allowed range of %d to %d\n",
				*value, lower_limit, upper_limit );

		} else {
			input_accepted = TRUE;
		}
	}

	return SUCCESS;
}

int
motor_get_long( FILE *file, char *prompt,
		int have_default, long default_value, long *value,
		long lower_limit, long upper_limit)
{
	char buffer[100];
	char default_string[50];
	int input_accepted, status;
	int count, string_length, buffer_length, whitespace_length;

	if ( have_default ) {
		sprintf( default_string, "%ld", default_value );
	} else {
		strcpy( default_string, "" );
	}

	input_accepted = FALSE;

	while ( ! input_accepted ) {

		buffer_length = sizeof(buffer) - 1;

		status = motor_get_string(file, prompt, default_string,
						&buffer_length, &buffer[0]);

		if ( status == FAILURE ) {
			return FAILURE;
		}

		string_length = strlen( buffer );

		whitespace_length = strspn( buffer, WHITESPACE );

		if ( whitespace_length >= string_length ) {
			if ( have_default ) {
				*value = default_value;
			} else {
				*value = 0;       /* The ultimate default. */
			}
		}

		count = sscanf( buffer, "%ld", value );

		if ( count == 0 ) {
			fprintf( file, "Non-numeric value entered.\n" );

		} else if ( *value < lower_limit || *value > upper_limit ) {
			fprintf( file,
		"Value %ld is outside the allowed range of %ld to %ld\n",
				*value, lower_limit, upper_limit );

		} else {
			input_accepted = TRUE;
		}
	}

	return SUCCESS;
}

int
motor_get_double( FILE *file, char *prompt,
		int have_default, double default_value, double *value,
		double lower_limit, double upper_limit)
{
	char buffer[100];
	char default_string[50];
	int input_accepted, status;
	int count, string_length, buffer_length, whitespace_length;

	if ( have_default ) {
		sprintf( default_string, "%g", default_value );
	} else {
		strcpy( default_string, "" );
	}

	input_accepted = FALSE;

	while ( ! input_accepted ) {

		buffer_length = sizeof(buffer) - 1;

		status = motor_get_string(file, prompt, default_string,
						&buffer_length, &buffer[0]);

		if ( status == FAILURE ) {
			return FAILURE;
		}

		string_length = strlen( buffer );

		whitespace_length = strspn( buffer, WHITESPACE );

		if ( whitespace_length >= string_length ) {
			if ( have_default ) {
				*value = default_value;
			} else {
				*value = 0;       /* The ultimate default. */
			}
		}

		count = sscanf( buffer, "%lf", value );

		if ( count == 0 ) {
			fprintf( file, "Non-numeric value entered.\n" );

		} else if ( *value < lower_limit || *value > upper_limit ) {
			fprintf( file,
		"Value %g is outside the allowed range of %g to %g\n",
				*value, lower_limit, upper_limit );

		} else {
			input_accepted = TRUE;
		}
	}

	return SUCCESS;
}

int
motor_get_string( FILE *file, char *prompt, char *default_string,
		int *string_length, char *string )
{
	int input_accepted;
	char real_prompt[250];

#if ( MX_CMDLINE_PROCESSOR == MX_CMDLINE_READLINE )
	char *ptr;
#else
	int i, c, new_string_length;
#endif

	if ( file == NULL ) {
		fprintf( stderr,
			"motor_get_string():  Error! (file == NULL)\n");
		return FAILURE;
	}

	if ( prompt == NULL ) {
		fprintf( file,
			"motor_get_string():  Error! (prompt == NULL)\n");
		return FAILURE;
	}

	if ( string_length == NULL ) {
		fprintf( file,
		"motor_get_string():  Error! (string_length == NULL)\n");
		return FAILURE;
	}

	if ( string == NULL ) {
		fprintf( file,
			"motor_get_string():  Error! (string == NULL)\n");
		return FAILURE;
	}

	if ( *string_length <= 0 ) {
		fprintf( file,
	"motor_get_string():  Error! Invalid length %d for string buffer.\n",
			*string_length );
		return FAILURE;
	}

	/* Construct the prompt the user will actually see. */

	/* EXTRA_CHARS is the number of characters added to the
	 * prompt for things like " (CR = " and so forth.
	 */

#define EXTRA_CHARS	11

	if ( default_string != NULL && strlen(default_string) > 0 ) {
		if ( strlen(prompt) + strlen(default_string) + EXTRA_CHARS
						> sizeof( real_prompt ) - 1 ) {
			fprintf( file, "motor_get_string():  Error! "
			"The prompt plus the default string are too long.\n");
			return FAILURE;
		}

		sprintf( real_prompt, "%s(CR = '%s') ",
			prompt, default_string );
	} else {
		if ( strlen(prompt) > sizeof(real_prompt) - 1 ) {
			fprintf( file, "motor_get_string():  Error! "
						"The prompt is too long.\n");
			return FAILURE;
		}

		strcpy( real_prompt, prompt );
	}

	/* Go into an input loop until we get a valid input. */

	input_accepted = FALSE;

#if ( MX_CMDLINE_PROCESSOR == MX_CMDLINE_READLINE )

	while ( ! input_accepted ) {
		/* We only use readline() on TTY devices.  This way, if
		 * standard input or output are redirected to pipes or
		 * some such, we will not get hung with readline() trying
		 * to make system calls that only work for TTY devices.
		 */

		if ( isatty( STDIN_FILENO ) && isatty( STDOUT_FILENO ) ) {
			ptr = readline( real_prompt );
	
			if ( ptr == NULL ) {
				return FAILURE;
			} else {
				if ( strlen(ptr) == 0 ) {
					if ( default_string != NULL &&
					  strlen(default_string) > 0 ) {
	
						strcpy( string, "" );
						strncat( string, default_string,
							*string_length );
					} else {
						strcpy( string, "" );
					}
				} else {
					strcpy( string, "" );
					strncat( string, ptr, *string_length );
				}
				*string_length = strlen(string);
				input_accepted = TRUE;
	
				free(ptr);
			}
		} else {
			fprintf( file, "%s", real_prompt );
			fflush( file );

			fgets( string, *string_length, stdin );

			if ( feof(input) || ferror(input) ) {
				*string = '\0';
				*string_length = 0;
			} else {
				/* Null terminate the string just in case
				 * it isn't.
				 */

				string[ *string_length - 1 ] = '\0';

				*string_length = strlen(string);

				/* Zap any trailing newline. */

				if ( string[ *string_length - 1 ] == '\n' ) {
					string[ *string_length - 1 ] = '\0';

					*string_length = strlen(string);
				}
			}
		}
	}

#else  /* MX_CMDLINE_PROCESSOR */

	while ( ! input_accepted ) {
		fprintf( file, "%s", real_prompt );
		fflush( file );

		for ( i = 0; i < *string_length; i++ ) {
			c = mx_getch();

			c &= 0xFF;	/* Mask to 8 bits. */

			if ( c == ESC ) {
				return FAILURE;
			}

			if ( c == CTRL_D ) {
				return FAILURE;
			}

			if ( c == CTRL_H ) {	/* backspace */
				if ( i > 0 ) {
					i--;

					string[i] = '\0';

					i--;

					fputc( CTRL_H, file );
					fputc( ' ', file );
					fputc( CTRL_H, file );
					fflush(file);
				}

				continue;	/* Cycle the for() loop. */
			}

			if ( c == '\n' || c == CR || c == LF ) {
				string[i] = '\0';

				fputc( '\n', file );
				fflush( file );

				break;		/* Exit the for() loop. */
			}

			if ( isprint(c) ) {
				fputc( c, file );
				fflush( file );

				string[i] = (char) c;
			}
		}

		if ( i < *string_length ) {
			new_string_length = i;
		} else {
			new_string_length = *string_length;
		}

		/* Null terminate the string, just in case it isn't already. */

		string[new_string_length] = '\0';

		/* Copy in the default string if the user didn't enter one. */

		if ( ( new_string_length <= 0 )
		  && ( default_string != NULL )
		  && ( strlen(default_string) > 0 ) )
		{
			strcpy( string, "" );
			strncat( string, default_string, *string_length );
			new_string_length = strlen(string);
		}

		*string_length = new_string_length;
		input_accepted = TRUE;
	}

#endif  /* MX_CMDLINE_PROCESSOR */

	return SUCCESS;
}

int
motor_get_string_from_list( FILE *file, char *prompt,
		int num_strings, int *min_length_array, char **string_array,
		int default_string_number, int *string_length,
		char *selected_string )
{
	const char fname[] = "motor_get_string_from_list()";

	char real_prompt[120];
	size_t buffer_left;
	int i, status, length, valid_input, output_buffer_length;
#if 0
	fprintf( file, "prompt = '%s'\n", prompt );
	fprintf( file, "num_strings = %d\n", num_strings );
	for ( i = 0; i < num_strings; i++ ) {
		fprintf( file,
		"min_length_array[%d] = %d, string_array[%d] = '%s'\n",
			i, min_length_array[i], i, string_array[i] );
	}
	fprintf( file, "initial *string_length = %d\n", *string_length );
#endif

	if ( file == NULL ) {
		fprintf(stderr, "%s: 'file' argument is NULL\n", fname);
		return FAILURE;
	}
	if ( string_length == NULL ) {
		fprintf(file, "%s: 'string_length' argument is NULL\n", fname);
		return FAILURE;
	}
	if ( selected_string == NULL ) {
		fprintf(file,"%s: 'selected_string' argument is NULL\n",fname);
		return FAILURE;
	}
	if ( min_length_array == NULL ) {
		fprintf(file,"%s: 'min_length_array' argument is NULL\n",fname);
		return FAILURE;
	}
	if ( string_array == NULL ) {
		fprintf(file,"%s: 'string_array' argument is NULL\n",fname);
		return FAILURE;
	}
	if ( num_strings <= 0 ) {
		fprintf(file,
		"%s: Illegal number of string arguments passed (value = %d)\n",
			fname, num_strings);
		return FAILURE;
	}
	if ( *string_length <= 0 ) {
		fprintf(file,
"%s: No room in output buffer for string response (*string_length = %d)\n",
			fname, *string_length );
		return FAILURE;
	}

	strcpy( real_prompt, "" );

	if ( prompt != NULL ) {
		strncat( real_prompt, prompt, sizeof( real_prompt ) - 1 );
	}

	output_buffer_length = *string_length;

	buffer_left = sizeof(real_prompt) - strlen(real_prompt) - 1;

	strncat( real_prompt, "[", buffer_left );

	for ( i = 0; i < num_strings; i++ ) {
		buffer_left = sizeof(real_prompt) - strlen(real_prompt) - 1;

		if ( i != 0 ) {
			strncat( real_prompt, ",", buffer_left );

			buffer_left = sizeof(real_prompt)
						- strlen(real_prompt) - 1;
		}

		if ( string_array[i] == NULL ) {
			fprintf(file,
			"%s: 'string_array[%d]' pointer is NULL.\n", fname, i);

			return FAILURE;
		}
		strncat( real_prompt, string_array[i], buffer_left );
	}
	buffer_left = sizeof(real_prompt) - strlen(real_prompt) - 1;

	strncat( real_prompt, "] -> ", buffer_left );

	valid_input = FALSE;

	while( valid_input == FALSE ) {
		*string_length = output_buffer_length;

		status = motor_get_string( file, real_prompt,
				string_array[ default_string_number ],
				string_length, selected_string );

		if ( status != SUCCESS )
			return status;

		length = strlen( selected_string );

		for ( i = 0; i < num_strings; i++ ) {
			if ( length >= min_length_array[i] ) {

				status = strncmp( selected_string,
					string_array[i], length );

				if ( status == 0 ) {
					valid_input = TRUE;

					break;	/* Exit the for() loop. */
				}
			}
		}
		if ( valid_input == FALSE ) {
			fprintf( file,
	"Invalid response '%s'.  Please try again or hit ctrl-D to abort.\n",
				selected_string );
		} else {
			strcpy( selected_string, "" );
			strncat( selected_string,
				string_array[i], output_buffer_length );
		}
	}

	*string_length = strlen( selected_string );

	return SUCCESS;
}

