/*
 * Name:    ms_prmca.c
 *
 * Purpose: Functions used to process MX MCA record events.
 *
 * Author:  William Lavender
 *
 *------------------------------------------------------------------------
 *
 * Copyright 2000 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>

#include "mxconfig.h"
#include "mx_util.h"
#include "mx_drivr.h"
#include "mx_rec.h"
#include "mx_mca.h"
#include "mx_sock.h"

#include "ms_serv.h"
#include "ms_prmca.h"

mx_status_type
mxsrv_setup_mca_process_functions( MX_RECORD *record_list_head )
{
	const char fname[] = "mxsrv_setup_mca_process_functions()";

	MX_RECORD *current_record;
	MX_RECORD_FIELD *record_field;
	MX_RECORD_FIELD *record_field_array;
	long i;

	MX_DEBUG(2,("%s invoked.", fname));

	/* The following is done just in case we were called with
	 * a record that isn't really the list head.
	 */

	record_list_head = record_list_head->list_head;

	/* Go through the record list looking for MCA records. */

	current_record = record_list_head->next_record;

	while ( current_record != record_list_head ) {

		if ( (current_record->mx_superclass == MXR_DEVICE)
		  && (current_record->mx_class == MXC_MULTICHANNEL_ANALYZER) ) {

			record_field_array
				= current_record->record_field_array;

			for ( i = 0;
				i < current_record->num_record_fields;
				i++ )
			{
				record_field = &record_field_array[i];

				switch( record_field->label_value ) {
				case MXLV_MCA_START:
				case MXLV_MCA_STOP:
				case MXLV_MCA_CLEAR:
				case MXLV_MCA_BUSY:
				case MXLV_MCA_CHANNEL_ARRAY:
				case MXLV_MCA_CURRENT_NUM_CHANNELS:
				case MXLV_MCA_ROI:
				case MXLV_MCA_ROI_INTEGRAL:
				case MXLV_MCA_CHANNEL_VALUE:
				case MXLV_MCA_REAL_TIME:
				case MXLV_MCA_LIVE_TIME:
					record_field->process_function
					    = mxsrv_mca_process_function;
					break;
				default:
					break;
				}
			}
		}
		current_record = current_record->next_record;
	}

	return MX_SUCCESSFUL_RESULT;
}

mx_status_type
mxsrv_mca_process_function( void *record_ptr,
			void *record_field_ptr, int operation )
{
	const char fname[] = "mxsrv_mca_process_function()";

	MX_RECORD *record;
	MX_RECORD_FIELD *record_field;
	MX_MCA *mca;
	char operation_string[30];
	long label_value;
	unsigned long ulong_value;
	double double_value;
	int busy;
	unsigned long roi_array[2];
	mx_status_type status;

	record = (MX_RECORD *) record_ptr;
	record_field = (MX_RECORD_FIELD *) record_field_ptr;

	switch ( operation ) {
	case MX_PROCESS_GET:
		strcpy( operation_string, "(MX_PROCESS_GET)" );
		break;
	case MX_PROCESS_PUT:
		strcpy( operation_string, "(MX_PROCESS_PUT)" );
		break;
	default:
		sprintf( operation_string, "(Operation %d)", operation );
		break;
	}

	label_value = record_field->label_value;

	MX_DEBUG(1,("%s: '%s.%s' %s (%ld)",
		fname, record->name, record_field->name,
		operation_string, label_value));

	/*
	 * A label value < 0 tells us that we do not need to do any
	 * further MX record processing. 
	 */

	if ( label_value < 0 ) {
		return MX_SUCCESSFUL_RESULT;
	}

	mca = (MX_MCA *) (record->record_class_struct);

	status = MX_SUCCESSFUL_RESULT;

	switch( operation ) {
	case MX_PROCESS_GET:
		switch( label_value ) {
		case MXLV_MCA_BUSY:
			status = mx_mca_is_busy( record, &busy );
			break;
		case MXLV_MCA_CHANNEL_ARRAY:
			status = mx_mca_read( record, NULL, NULL );
			break;
		case MXLV_MCA_CURRENT_NUM_CHANNELS:
			status = mx_mca_get_num_channels(
						record, &ulong_value );
			break;
		case MXLV_MCA_ROI:
			status = mx_mca_get_roi(record, roi_array);
			break;
		case MXLV_MCA_ROI_INTEGRAL:
			status = mx_mca_get_roi_integral(
						record, &ulong_value );
			break;
		case MXLV_MCA_CHANNEL_VALUE:
			status = mx_mca_get_channel( record,
						mca->channel_number,
						&ulong_value );
			break;
		case MXLV_MCA_REAL_TIME:
			status = mx_mca_get_real_time( record, &double_value );
			break;
		case MXLV_MCA_LIVE_TIME:
			status = mx_mca_get_live_time( record, &double_value );
			break;
		default:
			MX_DEBUG( 1,(
			    "%s: *** Unknown MX_PROCESS_GET label value = %ld",
				fname, label_value));
			break;
		}

		/* Now invoke the value changed callback. */

		if ( status.code == MXE_SUCCESS ) {
			status = mxsrv_invoke_callback_list(
					MXSF_VALUE_CHANGED_CALLBACK,
					record_field );
		}
		break;
	case MX_PROCESS_PUT:
		switch( label_value ) {
		case MXLV_MCA_START:
			status = mx_mca_start( record );
			break;
		case MXLV_MCA_STOP:
			status = mx_mca_stop( record );
			break;
		case MXLV_MCA_CLEAR:
			status = mx_mca_clear( record );
			break;
		case MXLV_MCA_ROI:
			status = mx_mca_set_roi( record, mca->roi );
			break;
		default:
			MX_DEBUG( 1,(
			    "%s: *** Unknown MX_PROCESS_PUT label value = %ld",
				fname, label_value));
			break;
		}

		/* Now invoke the value changed callback. */

		if ( status.code == MXE_SUCCESS ) {
			status = mxsrv_invoke_callback_list(
					MXSF_VALUE_CHANGED_CALLBACK,
					record_field );
		}
		break;
	default:
		return mx_error( MXE_ILLEGAL_ARGUMENT, fname,
			"Unknown operation code = %ld", operation );
		break;
	}

	return status;
}

