/*
 * Name:    ms_prmcs.c
 *
 * Purpose: Functions used to process MX MCS record events.
 *
 * Author:  William Lavender
 *
 *------------------------------------------------------------------------
 *
 * Copyright 2000 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>

#include "mxconfig.h"
#include "mx_util.h"
#include "mx_drivr.h"
#include "mx_rec.h"
#include "mx_mcs.h"
#include "mx_sock.h"

#include "ms_serv.h"
#include "ms_prmcs.h"

mx_status_type
mxsrv_setup_mcs_process_functions( MX_RECORD *record_list_head )
{
	const char fname[] = "mxsrv_setup_mcs_process_functions()";

	MX_RECORD *current_record;
	MX_RECORD_FIELD *record_field;
	MX_RECORD_FIELD *record_field_array;
	long i;

	MX_DEBUG( 2,("%s invoked.", fname));

	/* The following is done just in case we were called with
	 * a record that isn't really the list head.
	 */

	record_list_head = record_list_head->list_head;

	/* Go through the record list looking for MCS records. */

	current_record = record_list_head->next_record;

	while ( current_record != record_list_head ) {

		if ( (current_record->mx_superclass == MXR_DEVICE)
		  && (current_record->mx_class == MXC_MULTICHANNEL_SCALER) ) {

			record_field_array
				= current_record->record_field_array;

			for ( i = 0;
				i < current_record->num_record_fields;
				i++ )
			{
				record_field = &record_field_array[i];

				switch( record_field->label_value ) {
				case MXLV_MCS_START:
				case MXLV_MCS_STOP:
				case MXLV_MCS_CLEAR:
				case MXLV_MCS_BUSY:
				case MXLV_MCS_MODE:
				case MXLV_MCS_MEASUREMENT_TIME:
				case MXLV_MCS_MEASUREMENT_COUNTS:
				case MXLV_MCS_CURRENT_NUM_SCALERS:
				case MXLV_MCS_CURRENT_NUM_MEASUREMENTS:
				case MXLV_MCS_SCALER_INDEX:
				case MXLV_MCS_MEASUREMENT_INDEX:
				case MXLV_MCS_SCALER_DATA:
				case MXLV_MCS_MEASUREMENT_DATA:
				case MXLV_MCS_TIMER_DATA:
					record_field->process_function
					    = mxsrv_mcs_process_function;
					break;
				default:
					break;
				}
			}
		}
		current_record = current_record->next_record;
	}

	return MX_SUCCESSFUL_RESULT;
}

mx_status_type
mxsrv_mcs_process_function( void *record_ptr,
			void *record_field_ptr, int operation )
{
	const char fname[] = "mxsrv_mcs_process_function()";

	MX_RECORD *record;
	MX_RECORD_FIELD *record_field;
	MX_MCS *mcs;
	char operation_string[30];
	long label_value;
	int busy, mode;
	unsigned long ulong_value;
	double double_value;
	mx_status_type status;

	record = (MX_RECORD *) record_ptr;
	record_field = (MX_RECORD_FIELD *) record_field_ptr;

	switch ( operation ) {
	case MX_PROCESS_GET:
		strcpy( operation_string, "(MX_PROCESS_GET)" );
		break;
	case MX_PROCESS_PUT:
		strcpy( operation_string, "(MX_PROCESS_PUT)" );
		break;
	default:
		sprintf( operation_string, "(Operation %d)", operation );
		break;
	}

	label_value = record_field->label_value;

	MX_DEBUG( 1,("%s: '%s.%s' %s (%ld)",
		fname, record->name, record_field->name,
		operation_string, label_value));

	/*
	 * A label value < 0 tells us that we do not need to do any
	 * further MX record processing. 
	 */

	if ( label_value < 0 ) {
		return MX_SUCCESSFUL_RESULT;
	}

	mcs = (MX_MCS *) (record->record_class_struct);

	status = MX_SUCCESSFUL_RESULT;

	switch( operation ) {
	case MX_PROCESS_GET:
		switch( label_value ) {
		case MXLV_MCS_BUSY:
			status = mx_mcs_is_busy( record, &busy );
			break;
		case MXLV_MCS_MODE:
			status = mx_mcs_get_mode( record, &mode );
			break;
		case MXLV_MCS_MEASUREMENT_TIME:
			status = mx_mcs_get_measurement_time( record,
								&double_value );
			break;
		case MXLV_MCS_MEASUREMENT_COUNTS:
			status = mx_mcs_get_measurement_counts( record,
								&ulong_value );
			break;
		case MXLV_MCS_SCALER_DATA:
			status = mx_mcs_read_scaler( record,
					mcs->scaler_index, NULL, NULL );
			break;
		case MXLV_MCS_MEASUREMENT_DATA:
			status = mx_mcs_read_measurement( record,
					mcs->measurement_index, NULL, NULL );
			break;
		case MXLV_MCS_TIMER_DATA:
			status = mx_mcs_read_timer( record, NULL, NULL );
			break;
		default:
			MX_DEBUG( 1,(
			    "%s: *** Unknown MX_PROCESS_GET label value = %ld",
				fname, label_value));
			break;
		}

		/* Now invoke the value changed callback. */

		if ( status.code == MXE_SUCCESS ) {
			status = mxsrv_invoke_callback_list(
					MXSF_VALUE_CHANGED_CALLBACK,
					record_field );
		}
		break;
	case MX_PROCESS_PUT:
		switch( label_value ) {
		case MXLV_MCS_START:
			status = mx_mcs_start( record );
			break;
		case MXLV_MCS_STOP:
			status = mx_mcs_stop( record );
			break;
		case MXLV_MCS_CLEAR:
			status = mx_mcs_clear( record );
			break;
		case MXLV_MCS_MODE:
			status = mx_mcs_set_mode( record, mcs->mode );
			break;
		case MXLV_MCS_MEASUREMENT_TIME:
			status = mx_mcs_set_measurement_time( record,
						mcs->measurement_time );
			break;
		case MXLV_MCS_MEASUREMENT_COUNTS:
			status = mx_mcs_set_measurement_counts( record,
						mcs->measurement_counts );
			break;
		case MXLV_MCS_SCALER_INDEX:
			break;
		case MXLV_MCS_MEASUREMENT_INDEX:
			break;
		default:
			MX_DEBUG( 1,(
			    "%s: *** Unknown MX_PROCESS_PUT label value = %ld",
				fname, label_value));
			break;
		}

		/* Now invoke the value changed callback. */

		if ( status.code == MXE_SUCCESS ) {
			status = mxsrv_invoke_callback_list(
					MXSF_VALUE_CHANGED_CALLBACK,
					record_field );
		}
		break;
	default:
		return mx_error( MXE_ILLEGAL_ARGUMENT, fname,
			"Unknown operation code = %ld", operation );
		break;
	}

	return status;
}

